<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Http\Controllers\V1\Chemsite;

use App\Exceptions\ServerRunTimeException;
use App\Rhawn\Services\Chemsite\ChemsiteCompanyService;
use App\Services\Api\Chemsite\ChemsiteCustomerService;
use App\Services\Api\Chemsite\ChemsiteOrdersService;
use Illuminate\Http\Request;
use Jiannei\Response\Laravel\Support\Facades\Response;
use App\Http\Controllers\V1\Controller;
use App\Support\Traits\Helpers;

use OpenApi\Annotations\Post;
use OpenApi\Annotations\RequestBody;
use OpenApi\Annotations\MediaType;
use OpenApi\Annotations\Schema;
use OpenApi\Annotations\Property;
use OpenApi\Annotations\Response as AnnotationResponse;
use OpenApi\Annotations as OA;


class OrdersController extends Controller
{
    use Helpers;

    public function __construct(ChemsiteOrdersService $chemsiteOrdersService)
    {
       $this->chemsiteOrdersService = $chemsiteOrdersService;
       $this->controllerType = 'chemsite';
    }

    /**
     *
     * @Post(
     *     path="/openapi/chemsite/orders/createOrders",
     *     tags={"chemsite - 订单相关接口"},
     *     summary="生成销售订单",
     *     description="生成销售订单",
     *     @RequestBody(
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 required={"soCusPo", "soCaName","soCaMobile","soCaPhone","soCaProvince","soCaCity","soCaStreet","soNote","items"},
     *                 @Property(property="soCusPo", @Schema(type="string"),description="客户编号"),
     *                 @Property(property="soCaName", @Schema(type="string"),description="收货人"),
     *                 @Property(property="soCaMobile", @Schema(type="string"),description="收货人手机"),
     *                 @Property(property="soCaPhone", @Schema(type="string"),description="收货人电话"),
     *                 @Property(property="soCaProvince", @Schema(type="string"),description="收货人省份"),
     *                 @Property(property="soCaCity", @Schema(type="string"),description="收货人城市"),
     *                 @Property(property="soCaStreet", @Schema(type="string"),description="收货人区县"),
     *                 @Property(property="soNote", @Schema(type="string"),description="备注"),
     *                 @Property(property="items", type="array",
     *                      @OA\items(
     *                          @Property(property="pCode", @Schema(type="string"),description="产品code"),
     *                          @Property(property="brandId", @Schema(type="integer"),description="品牌id"),
     *                          @Property(property="num", @Schema(type="integer"),description="产品数量"),
     *                      ),description="当前页数"),
     *                 example={
                                "soCusPo": "2000000011111",
                                "soCaName": "张三",
                                "soCaMobile": "13321686555",
                                "soCaPhone": "61111111",
                                "soCaProvince": "上海",
                                "soCaCity": "上海市",
                                "soCaStreet": "望园路88888号888室",
                                "soNote": "测试",
                                "items": {{
                                            "pCode": "C10010520",
     *                                      "brandId" : "10",
                                            "num" : "1",
     *                                  }}
     *                         }
     *             )
     *         )
     *     ),
     *     @OA\Parameter(
     *         description="用户获取的token值",
     *         in="header",
     *         name="authorization",
     *         required=true,
     *         @OA\Schema(type="string"),
     *         @OA\Examples(example="authorization", value="bearerNWJiNDhkNzlmNjg0N2FlMmZiYjliZWM3NGVkNzIyMjNleUpsZUhCcGNtVWlPakUyTmpRMk1EazJORGNzSW1oaGMyZ2lPaUl5ZEhsc1JIQlhkWFpNUVdaWGJVRllJbjA9",summary=""),
     *     ),
     *     @AnnotationResponse(
     *         response="200",
     *         description="正常操作响应",
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 allOf={
     *                     @Schema(ref="#/components/schemas/ApiResponse"),
     *                     @Schema(
     *                         type="object",
     *                         @Property(property="data", ref="#/components/schemas/OrdersDetail")
     *                     )
     *                 }
     *             )
     *         )
     *     ),
     *     security={
     *         {"bearer_token":{}}
     *     }
     * )
     */
    public function createCustomerNewOrder(Request $request)
    {
        $companyCode = $request->get('company_code');
        $cusNumber = $request->get('cus_number');
        if(!$companyCode){
            return Response::ok('验证失败,参数缺少!');
        }
        $this->checkCustomerType($companyCode,$this->controllerType);

        $message = [
            'soCusPo.required' => "客户订单号必传",
            'soCaName.required' => "收货人必传",
            'soCaMobile.required' => "收货人手机必传",
            'soCaProvince.required' => "收货地省份必传",
            'soCaCity.required' => "收货地城市必传",
            'soCaStreet.required' => "收货地街道必传",
            'items.required' => "订单详情必传",
            'soPaymentMethod.required' => "开票方式必传",
        ];
        $this->validateRequest($request, $message);

        $requestParams = $this->formatKeysfromArray($request->all(),'toUnderScore');
        $requestParams = array_merge($requestParams,['customer_code' => $companyCode]);
        try{

            $companyInfo = app(ChemsiteCompanyService::class)->getCompanyThroughCusCode($companyCode);
            if(!$companyInfo){
                throw new ServerRunTimeException('经销商不存在');
            }
            $customer = app(ChemsiteCustomerService::class)->getCustomerByNumber($cusNumber);
            if(!$customer){
                throw new ServerRunTimeException('客户不存在');
            }
            if($customer['com_id'] != $companyInfo['com_id']){
                throw new ServerRunTimeException('不是该经销商的客户');
            }

            $requestParams['customer_code'] = $cusNumber;
            $requestParams['company_code'] = $companyCode;

            $orderDetail = $this->chemsiteOrdersService->createNewCustomerOrders($requestParams);
            if($orderDetail){
                $orderDetail = $this->formatKeysfromArray($orderDetail,'toCamelCase');
            }
            return Response::success($orderDetail,'操作成功');
        }catch(\Throwable $exception){
            return $this->returnErrorExecptionResponse($exception,'生成订单失败');
        }
    }

}
