<?php

namespace App\Services;

use App\Exceptions\NotFoundHttpException;
use App\Exceptions\ServerRunTimeException;
use App\Repositories\Contracts\FileManageRepository;
use App\Services\Traits\BaseService;
use App\Support\Traits\Helpers;
use Illuminate\Contracts\Filesystem\Factory as FilesystemFactory;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Sopamo\LaravelFilepond\Filepond;
use Symfony\Component\HttpFoundation\File\UploadedFile;

class FileService
{
    use Helpers, BaseService;

    public function __construct(FileManageRepository $fileManageRepository)
    {
        $this->setRepository($fileManageRepository);
    }

    /**
     * 文件上传
     * @param UploadedFile $file
     * @return array|null
     */
    public function uploadFile(UploadedFile $file)
    {
        $fileInfo = $this->getUploadFileInfo($file);
        $fileName = $fileInfo['name'].'.'.$fileInfo['extension'];

        $realPath = $this->getFileFolderPath().'/'.$fileName;
        if(
            $this->saveToFile($realPath,$file->getContent())
        ){
            $fileSave = $fileInfo;
            $fileSave['uniqueid'] = $this->getBase64Path($realPath);
            $fileSave['type'] = $this->isImageType($fileInfo) ? 'image' : 'file';
            $fileSave['path'] = $realPath;
            $saveResult = $this->saveFileRecord($fileSave);

            if($saveResult){
                $fileUrl = $this->getDownloadUrl($saveResult->uniqueid);
                return ['url' => $fileUrl, 'file' => $saveResult->uniqueid];
            }
        }
        return null;
    }

    public function getFileFolderPath($folderName = '')
    {
        $folder = $folderName;
        if(empty($folder)){
            $folder = Str::random();
        }
        return $this->getStoragePath($folder,'file');
    }

    /**
     * 保存图片
     * @param UploadedFile $file
     * @throws ServerRunTimeException
     */
    public function saveUploadImage(UploadedFile $file)
    {
        $newFile = $file->storeAs($this->getFileFolderPath(), $file->getClientOriginalName(), $this->getStorageDisk());
        if(!$newFile){
            throw new ServerRunTimeException('图片保存失败');
        }
        $imageStoragePath = $this->getBase64Path($newFile);

        $saveImage = $this->getUploadFileInfo($file);
        $saveImage['uniqueid'] = $imageStoragePath;
        $saveImage['path'] = $newFile;

        $result = $this->saveFileRecord($saveImage);
        if($result){
            $imageUrl = $this->getImageUrl($imageStoragePath);
            return ['url' => $imageUrl, 'image' => $imageStoragePath];
        }

        return null;
    }

    public function getFileSize($path): string
    {
        return ceil(filesize(Storage::path($path))/1000)."k";
    }


    public function getUploadFileInfo(UploadedFile $file): array
    {
        $fileInfo = [];
        $originalName = $file->getClientOriginalName();
        $pos = strrpos($file->getClientOriginalName(), '.');
        $originalName = false === $pos ? $originalName : substr($originalName, 0, $pos);

        $fileInfo['name'] = $originalName;
        $fileInfo['extension'] = $file->guessClientExtension();
        $fileInfo['mime_type'] = $file->getClientMimeType();
        $fileInfo['byte_size'] = $file->getSize();

        return $fileInfo;
    }

    public function getBase64Path($path)
    {
        return app(Filepond::class)->getServerIdFromPath($path);
    }

    public function getImageUrl($path, $type = 'image')
    {
        $path = $type.'/'.$path;
        return $this->getFileSystemDisk()->url($path);
    }

    public function getDownloadUrl($path)
    {
        $path = 'download/'.$path;
        return $this->getFileSystemDisk()->url($path);
    }

    public function download($path)
    {
        if (! trim($path)) {
            throw new NotFoundHttpException('文件无法读取，路径无效');
        }

        return $this->getFileSystemDisk()->download($this->loadPath($path));
    }

    public function downloadFile($path)
    {
        if (!trim($path)) {
            throw new NotFoundHttpException('文件无法读取，路径无效');
        }

        return $this->getFileSystemDisk()->download($this->loadPath($path));
    }

    public function getStoragePath($path, $type = 'image'): string
    {
        $localPath = 'upload/'.$type.'/'.date('Ymd',time()).'/';
        $storagePath = $localPath.$path;
        if(!Storage::exists($storagePath)){
            Storage::makeDirectory($storagePath);
        }
        return $storagePath;
    }

    public function getStorageDisk()
    {
        return config('filesystems.default');
    }

    public function getUploadImageMineType()
    {
        return ['image/apng','image/jpeg','image/png'];
    }

    public function isImageType($file)
    {
        $png = strpos($file['mime_type'], 'png');
        $jpg = strpos($file['mime_type'], 'jpeg');
        $gif = strpos($file['mime_type'], 'gif');
        if(!$png && !$jpg  && !$gif){
            return false;
        }
        return true;
    }

    public function isPdf($fileInfo)
    {
        return strpos($fileInfo['mime_type'], 'pdf');
    }

    public function isPpt($fileInfo)
    {
        return strpos($fileInfo['mime_type'], 'presentationml.presentation');
    }

    public function isExcel($fileInfo): bool
    {
        return strpos($fileInfo['mime_type'], 'ms-excel') || strpos($fileInfo['mime_type'], 'spreadsheetml.sheet');
    }

    public function isWord($fileInfo): bool
    {
        return strpos($fileInfo['mime_type'], 'msword')  || strpos($fileInfo['mime_type'], 'wordprocessingml.document');
    }

    public function loadPath($path)
    {
        $filePath = Crypt::decryptString($path);
        if(!$filePath){
            throw new NotFoundHttpException('图片或文件不存在');
        }
        return $filePath;
    }

    /**
     * 读取文件内容，返回base64
     * @param $path
     * @return mixed
     */
    public function loadFile($path)
    {
        return $this->getFileSystemDisk()->get($this->loadPath($path));
    }

    public function saveToFile($path,$content)
    {
        return $this->getFileSystemDisk()->put($path,$content);
    }

    public function getFileSystemDisk()
    {
        return app(FilesystemFactory::class)->disk($this->getStorageDisk());
    }


    public function updateFileExModelData($fileUid, $updateParams)
    {
        $file = $this->getRepository()->findWhere(['uniqueid' => $fileUid])->first();
        if(!$file){
            throw new ServerRunTimeException('上传文件记录未找到');
        }
        return $file->update($updateParams);
    }

    public function deleteFileByModelData($fileUid)
    {
        $file = $this->getRepository()->findWhere(['uniqueid' => $fileUid])->first();
        if($file){
            $this->removeFile($fileUid);
            $file->delete();
        }
        return true;
    }

    public function removeFile($path): bool
    {
        $fileInfo = $this->getRepository()->getFileByUniqueId($path);
        if(!$fileInfo){
            throw new NotFoundHttpException('图片或文件不存在');
        }
        $fileInfoArr = current($this->collectionToArray([$fileInfo]));
        $filePath = $fileInfoArr['path'];
        str_replace($fileInfoArr['name'].'.'.$fileInfoArr['extension'],'',$filePath);

        if($fileInfo->delete() && Storage::exists($filePath)/*$this->getRepository()->delete($fileInfo->id)*/){
           Storage::delete($fileInfoArr['path']) && Storage::deleteDirectory($filePath);
        }
        return true;
    }

    public function getFileListByUniqueId($uniqueIds)
    {
        $fileList = $this->getRepository()->findWhereIn('uniqueid', (array)$uniqueIds);
        if($fileList){
            return $this->collectionToArray($fileList);
        }
        return null;
    }

    public function getFileListByModelData($modelName, $modelId)
    {
        $fileList = $this->getRepository()->findWhere(['model_name' => $modelName, 'model_data_id' => $modelId]);
        if($fileList){
            return $this->collectionToArray($fileList);
        }
        return null;
    }

    /**
     * 验证上传的图片是否有效
     * @param $path
     * @param $validType
     * @return bool
     */
    public function validUploadFile($path, $validType = null): bool
    {
        $validResult = false;
        $fileInfo = $this->getFileListByUniqueId($path);
        if($fileInfo){
            $fileInfo = current($fileInfo);
            if(!is_null($validType)){
                $validType = is_array($validType) ? $validType : [$validType];
            }
            foreach($validType as $type){
                if(!$validResult){
                    switch($type){
                        case 'image': $this->isImageType($fileInfo) && $this->loadFile($path) ? $validResult = true : $validResult = false;break;
                        case 'svg': $this->isImageType($fileInfo) && $this->loadFile($path) ? $validResult = true : $validResult = false;break;
                        case 'pdf': $validResult = $this->isPdf($fileInfo);break;
                        case 'word': $validResult = $this->isWord($fileInfo);break;
                        case 'excel': $validResult = $this->isExcel($fileInfo);break;
                        case 'ppt': $validResult = $this->isPpt($fileInfo);break;
                        default: $validResult = strpos($fileInfo['mime_type'], $type) && $this->loadFile($path);
                    }
                }
                /*if($type == 'image'){
                    if($this->isImageType($fileInfo) && $this->loadFile($path)){
                        $validResult = true;
                        break;
                    }
                }else{
                    if($type == 'pdf'){
                        $validResult = $this->isPdf($fileInfo);
                        break;
                    }else if($type == 'excel'){
                        $validResult = $this->isExcel($fileInfo);
                        break;
                    }else{
                        $validResult = strpos($fileInfo['mime_type'], $type) && $this->loadFile($path);
                    }
                }*/
            }
        }
        return $validResult;
    }

    public function saveFileRecord($fileSaveParams)
    {
        return $this->getRepository()->saveUplodFiles($fileSaveParams);
    }


}
